@@ -199,11 +199,13 @@ def generate_tasks(layer_path, mip, roi_start, chunk_size, grid_size, queue_name
               help='Neuroglancer precomputed block compression algorithm.')
 @click.option('--voxel-size', '-v', type=int, nargs=3, default=(40, 4, 4),
               help='voxel size or resolution of mip 0 image.')
+@click.option('--overwrite-info/--no-overwrite-info', default=False,
+              help='normally we should avoid overwriting info file to avoid errors.')
 @generator
 def setup_env(volume_start, volume_stop, volume_size, layer_path, max_ram_size,
               output_patch_size, input_patch_size, channel_num, dtype, 
               output_patch_overlap, crop_chunk_margin, mip, thumbnail_mip, max_mip,
-              queue_name, visibility_timeout, thumbnail, encoding, voxel_size):
+              queue_name, visibility_timeout, thumbnail, encoding, voxel_size, overwrite_info):
     """Prepare storage info files and produce tasks."""
     assert not (volume_stop is None and volume_size is None)
     if isinstance(volume_start, tuple):
@@ -288,6 +290,9 @@ def setup_env(volume_start, volume_stop, volume_size, layer_path, max_ram_size,
     input_chunk_size = tuple(ocs + ccm*2 + ips - ops for ocs, ccm, ips, ops in zip(
         output_chunk_size, crop_chunk_margin, 
         input_patch_size, output_patch_size))
+    
+    expand_margin_size = tuple((ics - ocs) // 2 for ics, ocs in zip(
+        input_chunk_size, output_chunk_size))
 
     input_chunk_start = tuple(vs - ccm - (ips-ops)//2 for vs, ccm, ips, ops in zip(
         volume_start, crop_chunk_margin, input_patch_size, output_patch_size))
@@ -299,6 +304,8 @@ def setup_env(volume_start, volume_stop, volume_size, layer_path, max_ram_size,
     print('\ninput chunk size: ', input_chunk_size)
     print('input volume start: ', input_chunk_start)
     print('output chunk size: ', output_chunk_size)
+    print('cutout expand margin size: ', expand_margin_size)
+
     print('output volume start: ', volume_start)
     print('block size: ', block_size)
     print('RAM size of each block: ', 
@@ -307,12 +314,14 @@ def setup_env(volume_start, volume_stop, volume_size, layer_path, max_ram_size,
           np.prod(output_chunk_size)/np.prod(patch_num)/np.prod(output_patch_size))
    
     if not state['dry_run']:
-        print('\ncheck that we are not overwriting existing info file.')
         storage = SimpleStorage(layer_path)
-        assert not storage.exists('info')
         thumbnail_layer_path = os.path.join(layer_path, 'thumbnail')
         thumbnail_storage = SimpleStorage(thumbnail_layer_path)
-        assert not thumbnail_storage.exists('info')
+
+        if not overwrite_info:
+            print('\ncheck that we are not overwriting existing info file.')
+            assert not storage.exists('info')
+            assert not thumbnail_storage.exists('info')
 
         print('create and upload info file to ', layer_path)
         # Note that cloudvolume use fortran order rather than C order
@@ -464,33 +473,26 @@ def agglomerate(tasks, name, threshold, aff_threshold_low, aff_threshold_high,
               type=str,
               default='create-chunk',
               help='name of operator')
-@click.option('--size',
-              type=int,
-              nargs=3,
-              default=(64, 64, 64),
-              help='the size of created chunk')
+@click.option('--size', '-s',
+              type=int, nargs=3, default=(64, 64, 64), help='the size of created chunk')
 @click.option('--dtype',
               type=click.Choice(
                   ['uint8', 'uint32', 'uint16', 'float32', 'float64']),
-              default='uint8',
-              help='the data type of chunk')
+              default='uint8', help='the data type of chunk')
+@click.option('--all-zero/--not-all-zero', default=False, help='all zero or not.')
 @click.option('--voxel-offset',
-              type=int,
-              nargs=3,
-              default=(0, 0, 0),
-              help='offset in voxel number.')
-@click.option('--output-chunk-name',
-              '-o',
-              type=str,
-              default="chunk",
-              help="name of created chunk")
+              type=int, nargs=3, default=(0, 0, 0), help='offset in voxel number.')
+@click.option('--output-chunk-name', '-o',
+              type=str, default="chunk", help="name of created chunk")
 @operator
-def create_chunk(tasks, name, size, dtype, voxel_offset, output_chunk_name):
+def create_chunk(tasks, name, size, dtype, voxel_offset, all_zero, output_chunk_name):
     """Create a fake chunk for easy test."""
     print("creating chunk: ", output_chunk_name)
     for task in tasks:
         task[output_chunk_name] = Chunk.create(
-            size=size, dtype=np.dtype(dtype), voxel_offset=voxel_offset)
+            size=size, dtype=np.dtype(dtype), 
+            all_zero = all_zero,
+            voxel_offset=voxel_offset)
         yield task
 
 
@@ -626,6 +628,24 @@ def delete_task_in_queue(tasks, name):
                 task_handle, queue.queue_name))
 
 
+@main.command('delete-chunk')
+@click.option('--name', type=str, default='delete-var', help='delete variable/chunk in task')
+@click.option('--chunk-name', '-c',
+              type=str, required=True, help='the chunk name need to be deleted')
+@operator
+def delete_chunk(tasks, name, chunk_name):
+    """Delete a Chunk in task to release RAM"""
+    for task in tasks:
+        handle_task_skip(task, name)
+        if task['skip']:
+            print('skip deleting ', chunk_name)
+        else:
+            if state['verbose']:
+                print('delete chunk: ', chunk_name)
+            del task[chunk_name]
+            yield task
+ 
+
 @main.command('cutout')
 @click.option('--name',
               type=str, default='cutout', help='name of this operator')
@@ -783,6 +803,10 @@ def log_summary(log_dir, output_size):
 @main.command('normalize-section-contrast')
 @click.option('--name', type=str, default='normalize-section-contrast',
               help='name of operator.')
+@click.option('--input-chunk-name', '-i',
+              type=str, default=DEFAULT_CHUNK_NAME, help='input chunk name')
+@click.option('--output-chunk-name', '-o',
+              type=str, default=DEFAULT_CHUNK_NAME, help='output chunk name')
 @click.option('--levels-path', '-p', type=str, required=True,
               help='the path of section histograms.')
 @click.option('--lower-clip-fraction', '-l', type=float, default=0.01, 
@@ -794,7 +818,8 @@ def log_summary(log_dir, output_size):
 @click.option('--maxval', type=int, default=255,
               help='the maximum intensity of transformed chunk.')
 @operator
-def normalize_contrast_contrast(tasks, name, levels_path, lower_clip_fraction,
+def normalize_contrast_contrast(tasks, name, input_chunk_name, output_chunk_name, 
+                                levels_path, lower_clip_fraction,
                                 upper_clip_fraction, minval, maxval):
     """Normalize the section contrast using precomputed histograms."""
     
@@ -808,7 +833,7 @@ def normalize_contrast_contrast(tasks, name, levels_path, lower_clip_fraction,
         handle_task_skip(task, name)
         if not task['skip']:
             start = time()
-            task['chunk'] = state['operators'][name](task['chunk'])
+            task[output_chunk_name] = state['operators'][name](task[input_chunk_name])
             task['log']['timer'][name] = time() - start
         yield task
 
@@ -818,6 +843,10 @@ def normalize_contrast_contrast(tasks, name, levels_path, lower_clip_fraction,
               type=str,
               default='normalize-section-mu',
               help='name of operator.')
+@click.option('--input-chunk-name', '-i',
+              type=str, default=DEFAULT_CHUNK_NAME, help='input chunk name')
+@click.option('--output-chunk-name', '-o',
+              type=str, default=DEFAULT_CHUNK_NAME, help='output chunk name')
 @click.option('--nominalmin',
               type=float,
               default=None,
@@ -831,7 +860,8 @@ def normalize_contrast_contrast(tasks, name, levels_path, lower_clip_fraction,
               default=False,
               help='clip transformed values to be within the target range.')
 @operator
-def normalize_section_shang(tasks, name, nominalmin, nominalmax, clipvalues):
+def normalize_section_shang(tasks, name, input_chunk_name, output_chunk_name, 
+                            nominalmin, nominalmax, clipvalues):
     """Normalize voxel values based on slice min/max within the chunk, Shang's method.
     The transformed chunk has floating point values.
     """
@@ -846,7 +876,7 @@ def normalize_section_shang(tasks, name, nominalmin, nominalmax, clipvalues):
         handle_task_skip(task, name)
         if not task['skip']:
             start = time()
-            task['chunk'] = state['operators'][name](task['chunk'])
+            task[output_chunk_name] = state['operators'][name](task[input_chunk_name])
             task['log']['timer'][name] = time() - start
         yield task
 
@@ -950,9 +980,9 @@ def copy_var(tasks, name, from_name, to_name):
 @click.option('--dtype', '-d', type=click.Choice(['float32', 'float16']),
               default='float32', help='numerical precision.')
 @click.option('--framework', '-f',
-              type=click.Choice(['identity', 'pznet', 'pytorch',
+              type=click.Choice(['general', 'identity', 'pznet', 'pytorch',
                                  'pytorch-multitask']),
-              default='pytorch-multitask', help='inference framework')
+              default='general', help='inference framework')
 @click.option('--batch-size', '-b',
               type=int, default=1, help='mini batch size of input patch.')
 @click.option('--bump', type=click.Choice(['wu', 'zung']), default='wu',
@@ -989,6 +1019,7 @@ def inference(tasks, name, convnet_model, convnet_weight_path, input_patch_size,
         mask_myelin_threshold=mask_myelin_threshold,
         dry_run=state['dry_run'],
         verbose=state['verbose']) as inferencer:
+        
         state['operators'][name] = inferencer 
 
         for task in tasks:
@@ -1009,10 +1040,12 @@ def inference(tasks, name, convnet_model, convnet_weight_path, input_patch_size,
 
 @main.command('mask')
 @click.option('--name', type=str, default='mask', help='name of this operator')
-@click.option('--volume-path',
-              type=str,
-              required=True,
-              help='mask volume path')
+@click.option('--input-chunk-name', '-i',
+              type=str, default=DEFAULT_CHUNK_NAME, help='input chunk name')
+@click.option('--output-chunk-name', '-o',
+              type=str, default=DEFAULT_CHUNK_NAME, help='output chunk name')
+@click.option('--volume-path', '-v',
+              type=str, required=True, help='mask volume path')
 @click.option('--mip', type=int, default=5, help='mip level of mask')
 @click.option('--inverse/--no-inverse',
               default=False,
@@ -1028,7 +1061,8 @@ def inference(tasks, name, convnet_model, convnet_weight_path, input_patch_size,
               'check all zero will return boolean result.')
 @click.option('--skip-to', type=str, default='save', help='skip to a operator')
 @operator
-def mask(tasks, name, volume_path, mip, inverse, fill_missing, check_all_zero, skip_to):
+def mask(tasks, name, input_chunk_name, output_chunk_name, volume_path, 
+         mip, inverse, fill_missing, check_all_zero, skip_to):
     """Mask the chunk. The mask could be in higher mip level and we
     will automatically upsample it to the same mip level with chunk.
     """
@@ -1053,7 +1087,7 @@ def mask(tasks, name, volume_path, mip, inverse, fill_missing, check_all_zero, s
                     print(yellow(f'the mask of {name} is all zero, will skip to {skip_to}'))
                 task['skip_to'] = skip_to
             else:
-                task['chunk'] = state['operators'][name](task['chunk'])
+                task[output_chunk_name] = state['operators'][name](task[input_chunk_name])
             # Note that mask operation could be used several times,
             # this will only record the last masking operation
             task['log']['timer'][name] = time() - start
@@ -1094,26 +1128,29 @@ def crop_margin(tasks, name, margin_size,
 
 @main.command('mesh')
 @click.option('--name', type=str, default='mesh', help='name of operator')
-@click.option('--chunk-name', '-c',
-              type=str, default='chunk', help='name of chunk needs to be meshed.')
-@click.option('--mip', '-c',
+@click.option('--input-chunk-name', '-i',
+              type=str, default=DEFAULT_CHUNK_NAME, help='name of chunk needs to be meshed.')
+@click.option('--mip', '-m',
     type=int, default=None, help='mip level of segmentation chunk.')
 @click.option('--voxel-size', '-v', type=int, nargs=3, default=None, callback=default_none, 
     help='voxel size of the segmentation. zyx order.')
 @click.option('--output-path', '-o', type=str, default='file:///tmp/mesh/', 
     help='output path of meshes, follow the protocol rule of CloudVolume. \
               The path will be adjusted if there is a info file with precomputed format.')
-@click.option('--output-format', '-f', type=click.Choice(['ply', 'obj', 'precomputed']), default='precomputed', 
-    help='output format, could be one of ply|obj|precomputed.')
-@click.option('--simplification-factor', '-s', type=int, default=100, help='mesh simplification factor.')
-@click.option('--max-simplification-error', '-m', type=int, default=40, help='max simplification error.')
+@click.option('--output-format', '-t', type=click.Choice(['ply', 'obj', 'precomputed']), 
+              default='precomputed', help='output format, could be one of ply|obj|precomputed.')
+@click.option('--simplification-factor', '-f', type=int, default=100, 
+              help='mesh simplification factor.')
+@click.option('--max-simplification-error', '-e', type=int, default=40, 
+              help='max simplification error.')
 @click.option('--dust-threshold', '-d',
     type=int, default=None, help='do not mesh segments with voxel number less than threshold.')
-@click.option('--ids', '-i', type=str, default=None, 
-              help='a list of segment ids to mesh. This is for sparse meshing. The ids should be separated by comma without space, such as "34,56,78,90"')
+@click.option('--ids', type=str, default=None, 
+              help='a list of segment ids to mesh. This is for sparse meshing. ' + 
+              'The ids should be separated by comma without space, such as "34,56,78,90"')
 @click.option('--manifest/--no-manifest', default=False, help='create manifest file or not.')
 @operator
-def mesh(tasks, name, chunk_name, mip, voxel_size, output_path, output_format,
+def mesh(tasks, name, input_chunk_name, mip, voxel_size, output_path, output_format,
          simplification_factor, max_simplification_error, dust_threshold, 
          ids, manifest):
     """Perform meshing for segmentation chunk."""
@@ -1136,7 +1173,7 @@ def mesh(tasks, name, chunk_name, mip, voxel_size, output_path, output_format,
         handle_task_skip(task, name)
         if not task['skip']:
             start = time()
-            task[chunk_name] = state['operators'][name](task[chunk_name])
+            state['operators'][name]( task[input_chunk_name] )
             task['log']['timer'][name] = time() - start
         yield task
 
@@ -1245,6 +1282,21 @@ def save(tasks, name, volume_path, input_chunk_name, upload_log, nproc, create_t
             task['output_volume_path'] = volume_path
         yield task
 
+
+@main.command('channel-voting')
+@click.option('--name', type=str, default='channel-voting', help='name of operator')
+@click.option('--input-chunk-name', type=str, default=DEFAULT_CHUNK_NAME)
+@click.option('--output-chunk-name', type=str, default=DEFAULT_CHUNK_NAME)
+@operator
+def channel_voting(tasks, name, input_chunk_name, output_chunk_name):
+    """all channels vote to get a uint8 volume. The channel with max intensity wins."""
+    for task in tasks:
+        handle_task_skip(task, name)
+        if not task['skip']:
+            task[output_chunk_name] = task[input_chunk_name].channel_voting() 
+        yield task
+
+
 @main.command('view')
 @click.option('--name', type=str, default='view', help='name of this operator')
 @click.option('--image-chunk-name',
